package com.wizarpos.apidemo.action;

import com.cloudpos.DeviceException;
import com.cloudpos.OperationListener;
import com.cloudpos.OperationResult;
import com.cloudpos.POSTerminal;
import com.cloudpos.TimeConstants;
import com.cloudpos.fingerprint.Fingerprint;
import com.cloudpos.fingerprint.FingerprintDevice;
import com.cloudpos.fingerprint.FingerprintOperationResult;
import com.wizarpos.apidemoforunionpaycloudpossdk.R;
import com.wizarpos.mvc.base.ActionCallback;

import java.util.Map;

/**
 * create by rf.w 19-2-28上午10:24
 */
public class IsoFingerPrintAction extends ActionModel {
    private FingerprintDevice device = null;

    private int userID = 9;
    private int timeout = 10 * 1000;
    private static final int ISOFINGERPRINT_TYPE_DEFAULT = 0;
    private static final int ISOFINGERPRINT_TYPE_ISO2005 = 1;
    private static final int ISOFINGERPRINT_TYPE_ISO2015 = 2;

    @Override
    protected void doBefore(Map<String, Object> param, ActionCallback callback) {
        super.doBefore(param, callback);
        if (device == null) {
            device = (FingerprintDevice) POSTerminal.getInstance(mContext).getDevice("cloudpos.device.fingerprint");
        }
    }

    public void open(Map<String, Object> param, ActionCallback callback) {
        try {
            device.open(1);
            //清除指纹数据
            device.delAllFingers();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }


    public void listenForFingerprint(Map<String, Object> param, ActionCallback callback) {
        sendSuccessLog("");
        try {
            OperationListener listener = new OperationListener() {

                @Override
                public void handleResult(OperationResult arg0) {
                    if (arg0.getResultCode() == OperationResult.SUCCESS) {
                        sendSuccessLog2(mContext.getString(R.string.scan_fingerprint_success));
                    } else {
                        sendFailedLog2(mContext.getString(R.string.scan_fingerprint_fail));
                    }
                }
            };
            device.listenForFingerprint(listener, TimeConstants.FOREVER);
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void waitForFingerprint(Map<String, Object> param, ActionCallback callback) {
        sendSuccessLog("");
        try {
            FingerprintOperationResult operationResult = device.waitForFingerprint(TimeConstants.FOREVER);
            if (operationResult.getResultCode() == OperationResult.SUCCESS) {
                sendSuccessLog2(mContext.getString(R.string.scan_fingerprint_success));
            } else {
                sendFailedLog2(mContext.getString(R.string.scan_fingerprint_fail));
            }
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void enroll(Map<String, Object> param, ActionCallback callback) {
        try {
            callback.sendResponse(mContext.getString(R.string.press_fingerprint));//请按手指
            userID++;
            device.enroll(userID, timeout);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void verifyAgainstUserId(Map<String, Object> param, ActionCallback callback) {
        callback.sendResponse(mContext.getString(R.string.press_fingerprint));//请按手指
        try {
            device.verifyAgainstUserId(userID, timeout);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void verifyAll(Map<String, Object> param, ActionCallback callback) {
        try {
            callback.sendResponse(mContext.getString(R.string.press_fingerprint));//请按手指
            device.verifyAll(timeout);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void getId(Map<String, Object> param, ActionCallback callback) {
        try {
            device.getId();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    Fingerprint fingerprint1;

    public void getFingerprint(Map<String, Object> param, ActionCallback callback) {
        callback.sendResponse(mContext.getString(R.string.press_fingerprint));//请按手指
        try {
            fingerprint1 = device.getFingerprint(ISOFINGERPRINT_TYPE_DEFAULT);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void verifyAgainstFingerprint(Map<String, Object> param, ActionCallback callback) {
        callback.sendResponse(mContext.getString(R.string.press_fingerprint));//请按手指
        try {
            device.verifyAgainstFingerprint(fingerprint1, timeout);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void storeFeature(Map<String, Object> param, ActionCallback callback) {
        try {
            device.storeFeature(userID, fingerprint1);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void match(Map<String, Object> param, ActionCallback callback) {
        callback.sendResponse(mContext.getString(R.string.press_fingerprint));//请按手指
        try {
            Fingerprint fingerprint2 = device.getFingerprint(ISOFINGERPRINT_TYPE_DEFAULT);
            device.match(fingerprint1, fingerprint2);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }


    public void convertFormat(Map<String, Object> param, ActionCallback callback) {
        try {
            Fingerprint fingerprint2 = new Fingerprint();
            byte[] arryFea = new byte[8192];
            fingerprint2.setFeature(arryFea);
            device.convertFormat(fingerprint1, ISOFINGERPRINT_TYPE_DEFAULT, fingerprint2, ISOFINGERPRINT_TYPE_ISO2005);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void listAllFingersStatus(Map<String, Object> param, ActionCallback callback) {
        try {
            device.listAllFingersStatus();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void delFinger(Map<String, Object> param, ActionCallback callback) {
        try {
            device.delFinger(userID);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void delAllFingers(Map<String, Object> param, ActionCallback callback) {
        try {
            device.delAllFingers();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void close(Map<String, Object> param, ActionCallback callback) {
        try {
            device.close();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }
}
