
package com.wizarpos.apidemo.action;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.os.Looper;

import com.cloudpos.DeviceException;
import com.cloudpos.POSTerminal;
import com.cloudpos.jniinterface.RFCardInterface;
import com.cloudpos.printer.Format;
import com.cloudpos.printer.PrinterDevice;
import com.wizarpos.apidemo.common.Common;
import com.wizarpos.apidemoforunionpaycloudpossdk.R;
import com.wizarpos.mvc.base.ActionCallback;
import com.wizarpos.mvc.impl.ActionCallbackImpl;

import java.util.Map;
import java.util.logging.Logger;

public class PrinterAction extends ActionModel {

    private PrinterDevice device = null;

    @Override
    protected void doBefore(Map<String, Object> param, ActionCallback callback) {
        super.doBefore(param, callback);
        if (device == null) {
            device = (PrinterDevice) POSTerminal.getInstance(mContext)
                    .getDevice("cloudpos.device.printer");
        }
    }

    public void open(Map<String, Object> param, ActionCallback callback) {
        try {
            device.open();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void printText(Map<String, Object> param, ActionCallback callback) {
        try {
//            device.printText("This is a test\n");
            Format format = new Format();
            format.setParameter(Format.FORMAT_FONT_SIZE, Format.FORMAT_FONT_SIZE_SMALL);
            device.printText(format, "This is a test\n");
            device.printText(format, "This is a test\n");
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void sendESCCommand(Map<String, Object> param, ActionCallback callback) {
        byte[] command = new byte[]{
                (byte) 0x12, (byte) 0x54
        };
        try {
            device.sendESCCommand(command);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void queryStatus(Map<String, Object> param, ActionCallback callback) {
        try {
            int status = device.queryStatus();
            sendSuccessLog(mContext.getString(R.string.operation_succeed)
                    + " Status: "
                    + (status == PrinterDevice.STATUS_OUT_OF_PAPER ? "out of paper"
                    : "paper exists"));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void cutPaper(Map<String, Object> param, ActionCallback callback) {
        try {
            device.cutPaper();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void printBitmap(Map<String, Object> param, ActionCallback callback) {
        Bitmap bitmap = null;
        try {
            bitmap = BitmapFactory.decodeStream(mContext.getResources().getAssets()
                    .open("printer_barcode_low.png"));
        } catch (Exception e1) {
            e1.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
        try {
            Format format = new Format();
            format.setParameter(Format.FORMAT_ALIGN, Format.FORMAT_ALIGN_RIGHT);
            device.printBitmap(format, bitmap);
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void printHtml(final Map<String, Object> param, final ActionCallback callback) {
        try {
            final String htmlContent = "The 1st mobile payment solution based on WAP in the world in 2001.\n" +
                    "\n" +
                    "<h1>The 1st PCI certified handheld EFT POS in the world in 2005.</h1>\n" +
                    "\n" +
                    "The 1st secure embedded Java platform accredited by the payment industry.\n" +
                    "\n" +
                    "<h2>The 1st GlobalPlatform & STIP compliant POS product in mass production in 2006.</h2>\n" +
                    "\n" +
                    "The 1st cloud based and GlobalPlatform compliant device management and multi-application provisioning system app store for POS terminal.\n" +
                    "\n" +
                    "The 1st generation e-commerce, payment gateway and e-wallet products based on Secure Electronic Transaction (SET) protocol in 1997.\n" +
                    "\n" +
                    "The 1st generation key management system for EMV chip card in China in 1998.\n" +
                    "\n" +
                    "<h5>The 1st digital certificate system CA in China.<h5>\n" +
                    "\n" +
                    "The 1st online merchant in China.\n" +
                    "\n" +
                    "The 1st internet banking and payment system in China\n\n\n";
            new Handler(Looper.getMainLooper()).post(new Runnable() {
                @Override
                public void run() {
                    try {
                        device.printHTML(mContext, htmlContent, null);
                        sendSuccessLog(mContext.getString(R.string.operation_succeed));
                    } catch (Exception e) {
                        sendFailedLog(mContext.getString(R.string.operation_failed));
                    }
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }


    public void printBarcode(Map<String, Object> param, ActionCallback callback) {
        try {
            Format format = new Format();
            format.setParameter("HRI-location", "up");
            //0, 01234567896
            //1, "04310000526"
            //
            device.printBarcode(format, PrinterDevice.BARCODE_CODE128, "000023");
            device.printText("\n\n\n");
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void cancelRequest(Map<String, Object> param, ActionCallback callback) {
        try {
            device.cancelRequest();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }

    public void close(Map<String, Object> param, ActionCallback callback) {
        try {
            device.close();
            sendSuccessLog(mContext.getString(R.string.operation_succeed));
        } catch (DeviceException e) {
            e.printStackTrace();
            sendFailedLog(mContext.getString(R.string.operation_failed));
        }
    }
}
